package com.acuant.sampleapp

import android.app.AlertDialog
import android.content.Intent
import android.graphics.BitmapFactory
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import android.view.View
import android.widget.*
import com.acuant.acuantcamera.camera.AcuantCameraActivity
import com.flutter_plugin_kyc.KYCResult
import com.flutter_plugin_kyc.MyApplication
import com.flutter_plugin_kyc.R
import com.trulioo.normalizedapi.model.DataFields
import com.trulioo.normalizedapi.model.Document
import com.trulioo.normalizedapi.model.PersonInfo
import com.trulioo.normalizedapi.model.VerifyRequest

class TruliooConfirmationActivity : AppCompatActivity() {

    var firstNameBox:EditText? = null
    var lastNameBox:EditText? = null
    var countryDropDown:Spinner? = null
    var documentTypeTextBox:TextView? = null
    var cancelButton: Button? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.activity_trulioo_confirmation)

        var frontImage = TruliooInformationStorage.frontImageFile
        var backImage = TruliooInformationStorage.backImageFile
        var selfieImage = TruliooInformationStorage.selfieImageFile

        firstNameBox = findViewById<EditText>(R.id.firstNameBox)
        lastNameBox = findViewById<EditText>(R.id.lastNameBox)
        cancelButton = findViewById(R.id.cancelButton)
        cancelButton?.setOnClickListener {
            cancelClicked()
        }

        // supported countryCodes/ documentTypes can be fetch from GlobalGateway server
        // please see the details on the documentation page
        // https://developer.trulioo.com/docs/document-verification-step-3-get-country-codes
        // https://developer.trulioo.com/docs/document-verification-step-4-get-document-types
        countryDropDown = findViewById<Spinner>(R.id.countryDropDown)
        documentTypeTextBox = findViewById<TextView>(R.id.documentTypeBox)
        documentTypeTextBox?.text = TruliooInformationStorage.cardType

        val frontImageBox = findViewById<ImageView>(R.id.frontImage)
        if(frontImage != null){
            frontImageBox.setImageBitmap(BitmapFactory.decodeFile(frontImage.path))
            frontImageBox.scaleType = ImageView.ScaleType.FIT_CENTER
        }

        val backImageBox = findViewById<ImageView>(R.id.backImage)
        if(backImage != null){
            backImageBox.setImageBitmap(BitmapFactory.decodeFile(backImage.path))
            backImageBox.scaleType = ImageView.ScaleType.FIT_CENTER
        }

        val selfieImageBox = findViewById<ImageView>(R.id.selfieImage)
        if(selfieImage != null){
            selfieImageBox.setImageBitmap(BitmapFactory.decodeFile(selfieImage.path))
            selfieImageBox.scaleType = ImageView.ScaleType.FIT_CENTER
        }
    }

    fun confirmClicked(view: View) {
        var firstName = firstNameBox?.text.toString().trim()
        var lastName = lastNameBox?.text.toString().trim()
        TruliooInformationStorage.firstName = firstName
        TruliooInformationStorage.lastName = lastName
        TruliooInformationStorage.countryCode = countryDropDown?.getSelectedItem().toString()

        if(firstName.isNullOrBlank() || lastName.isNullOrBlank()){
            val alert = AlertDialog.Builder(this@TruliooConfirmationActivity)
            alert.setTitle("Error")
            alert.setMessage("Please Enter Both First And Last Name")
            alert.setPositiveButton("OK") { dialog, whichButton ->
                dialog.dismiss()

            }
            alert.show()
        }
        else{
            val alert = AlertDialog.Builder(this@TruliooConfirmationActivity)
            alert.setTitle("Message")
            alert.setMessage("Verify: "+ firstName + " " + lastName)
            alert.setPositiveButton("OK") { dialog, whichButton ->
                dialog.dismiss()
                sendRequest()
            }
            alert.setNegativeButton("Cancel"){dialog, whichButton ->
                dialog.dismiss()
            }
            alert.show()

        }
    }

    fun sendRequest(){
        /*val result = Intent(
                this,
                TruliooResultActivity::class.java
        )
        startActivity(result)*/
        var request = buildRequest()
        MyApplication.objMSG = KYCResult(request);
        if(MyApplication.objMSG != null){
            val resultMain = Intent(
                    this,
                    MainActivity::class.java
            )
            resultMain.putExtra("msg", MyApplication.KYC_MSG)
            startActivity(resultMain)
        }
    }
    fun buildRequest(): VerifyRequest {
        val document = Document()

        document.documentFrontImage(TruliooInformationStorage.frontImageFile.readBytes())
        document.documentType(TruliooInformationStorage.cardType)

        if(TruliooInformationStorage.backImageFile != null){
            document.documentBackImage(TruliooInformationStorage.backImageFile.readBytes())
        }

        if(TruliooInformationStorage.selfieImageFile != null){
            document.livePhoto(TruliooInformationStorage.selfieImageFile.readBytes())
        }

        return VerifyRequest()
                .acceptTruliooTermsAndConditions(true)
                .configurationName("Identity Verification")
                .countryCode(TruliooInformationStorage.countryCode)
                .dataFields(DataFields()
                        .personInfo(PersonInfo()
                                .firstGivenName(TruliooInformationStorage.firstName)
                                .firstSurName(TruliooInformationStorage.lastName))
                        .document(document))
    }
    fun cancelClicked() {
        returnToMainPage()
    }

    fun returnToMainPage(){
        val result = Intent(
                this,
                MainActivity::class.java
        )
        startActivity(result)
    }

    override fun onBackPressed() {
        TruliooInformationStorage.cleanup()
        returnToMainPage()
    }

}
